@@ -115,6 +115,12 @@ void reconnect()
 	if(mystate->reconnect == OER_RECONNECT_NONE) {
 		return;
 	}
+	/* this is a workaround for the situation where admin changes oer+MySQL's
+           nick to a taken one and then jumps/reconnects, oer+MySQL should at least
+           try to get the one the admin specified or fall back to alternative */
+        if(strcasecmp(mystate->nick, mystate->getnick)) {
+                strncpy(mystate->nick, mystate->getnick, NICKLEN);
+        }
 	switch(mystate->reconnect) {
 	case OER_RECONNECT_ADMIN:
 		snprintf(stringbuffer, WRITE_BUFFER_LENGTH, "QUIT :%s\n", mystate->signoff);
@@ -363,7 +369,7 @@ int registerconnection()
         }
 	if(!mystate->current_server->registered.nick) {
                 /* NICK */
-                snprintf(stringbuffer, WRITE_BUFFER_LENGTH, "NICK %s\n", mystate->nick);
+		snprintf(stringbuffer, WRITE_BUFFER_LENGTH, "NICK %s\n", mystate->nick);
 #ifdef DEBUG
                 oer_debug(OER_DEBUG_INFO, "registerconnection->%s", stringbuffer);
 #endif
