@@ -165,6 +165,8 @@ enum WHICHCOMMAND_RETURN_VALUES {
         OER_WHICHCOMMAND_DATE,
         OER_WHICHCOMMAND_USERCOPY,
         OER_WHICHCOMMAND_CLEARBANS,
+        OER_WHICHCOMMAND_CYCLE,
+        OER_WHICHCOMMAND_BSTATS,
 	OER_WHICHCOMMAND_INVALID
 };
 
@@ -254,7 +256,7 @@ enum OER_FLAGS_TYPE {
 };
 
 #define OER_FLAGS_GLOBAL "dfglmnpqs"
-#define OER_FLAGS_CHANNEL "!aAbcDefFGklLmMnNoOpPqrsSuUtTvVwx"
+#define OER_FLAGS_CHANNEL "!aAbcDefFGklLmMnNoOpPqrRsSuUtTvVwx"
 #define OER_FLAGS_ADMIN "dn"
 #define OER_FLAGS_USER "!adfmovrx"
 
@@ -357,7 +359,7 @@ enum OER_FLAGS_TYPE {
 details possibly available on the oer+MySQL console"
 #define OER_REALNAME "http://oer.equnet.org/help.html"
 #define OER_UNKNOWN "unknown"
-#define OER_COPYRIGHT1 "oer+MySQL version 1.0-30, Copyright (C) 2000-2002 EQU <equ@equnet.org>"
+#define OER_COPYRIGHT1 "oer+MySQL version 1.0-31, Copyright (C) 2000-2002 EQU <equ@equnet.org>"
 #define OER_COPYRIGHT2 "oer+MySQL comes with ABSOLUTELY NO WARRANTY; for details see the\n\
 GNU General Public License (COPYING) provided with this distribution.\n\
 This is free software, and you are welcome to redistribute it under\n\
@@ -570,6 +572,7 @@ struct channel {
 	time_t last_quote;
 	time_t join_ts;
 	time_t userts;
+	time_t requestop;
 	struct join *joins;
 	struct part *parts;
 	struct pubmsg *pubmsgs;
