@@ -125,7 +125,7 @@ int main(int argc, char *argv[])
 	}
 #ifdef HAVE_LOCALE_H
         if((locptr = setlocale(LC_CTYPE, "")) == NULL) {
-                oer_debug(OER_DEBUG_FATAL, "main->setlocale() failed for %s\n");
+                oer_debug(OER_DEBUG_FATAL, "main->setlocale() failed\n");
                 exit(EXIT_FAILURE);
         }
 #endif
