@@ -463,8 +463,6 @@ void parseirc(char *serveroutput)
                         }
                         return;
                 }
-                /* no, must be a nick change that failed */
-                strncpy(mystate->nick, params, NICKLEN);
                 return;
         }
 	if(!strcasecmp(command, "437")) {
@@ -759,6 +757,16 @@ void parseirc(char *serveroutput)
                         return;
                 }
                 strncpy(userhost, prefix + 1 + tpos, USERHOSTLEN);
+		if(isme(nick)) {
+                        /* oer+MySQL changed it's nick either because it was
+                         asked to or it wasn't the primary nick and
+			 global flag "g" was set */
+#ifdef DEBUG
+                        oer_debug(OER_DEBUG_INFO, "nickchange->I changed my nick from %s to %s\n", nick, params + 1);
+#endif
+                        strncpy(mystate->nick, params + 1, NICKLEN);
+                        return;
+                }
 #ifdef DEBUG
                 oer_debug(OER_DEBUG_INFO, "nickchange->%s is now known as %s, changing on all channels\n", nick, params + 1);
 #endif
@@ -768,21 +776,19 @@ void parseirc(char *serveroutput)
                         sethostquerystatus(nick, 0);
                 }
                 /* nick doesn't provide the channel, we have to check all channels */
-                for(this = mystate->channels; this != NULL; this = this->next) {
-                        nickchange(this, nick, params + 1);
-                        /* we change all channels but check only those that are needed (why?) */
-                        if(isonchan(this, params + 1)) {
+		for(this = mystate->channels; this != NULL; this = this->next) {
+                        if(isonchan(this, nick)) {
                                 addnewnickchange(this, userhost);
-                                if(this->i_am_op) {
-                                        if(checkforbadnick(this, params + 1, userhost) || checkfornickflood(this, params + 1, userhost)) {
-                                                changetobek(this, params + 1, 1);
-                                        }
-                                } else {
-                                        /* this is a preventative measure. if a user tries
-                                           to avoid getting kicked on !lock by changing nick,
-                                           we will adapt */
-                                        changetobek(this, params + 1, 0);
+                                /* if a user tries to avoid getting kicked
+                                   on !lock by changing nick, we will adapt */
+                                changetobek(this, nick, 0);
+                                if(this->i_am_op && checkforbadnick(this, params + 1, userhost)) {
+                                        changetobek(this, nick, 1);
                                 }
+                                if(this->i_am_op && checkfornickflood(this, nick, userhost)) {
+					changetobek(this, nick, 1);
+				}
+                                nickchange(this, nick, params + 1);
                         }
                 }
                 return;
@@ -944,6 +950,11 @@ void parseirc(char *serveroutput)
                                 oer_debug(OER_DEBUG_NOISE, "parseirc(MODE)->OER_NTHMODE_DOP\n");
 #endif
                                 changeuser(this, target, -1, 0, -1, -1);
+				/* check if the last user on channel was de-opped, if
+				   so activate requestop */
+                                if(ischanopless(this)) {
+                                        this->requestop = mystate->now + 300;
+                                }
                                 if(isme(target)) {
                                         this->i_am_op = 0;
                                 }
@@ -951,6 +962,9 @@ void parseirc(char *serveroutput)
                                         /* allow Q/service to de-op */
                                         continue;
                                 }
+                                if(!this->i_am_op) {
+                                        continue;
+                                }
                                 if(haschanflags(this, "D") && isallowedop(this, target)) {
                                         /* re-op */
                                         mmode_new(this, mystate->now, "+o", target);
@@ -1110,7 +1124,7 @@ void parsemsg(char *nick, char *userhost, char *target, char *message)
 	int ppos;
 	int is_command;
 	struct channel *this;
-	char np[NICKLEN + 1];
+	char outstring[WRITE_BUFFER_LENGTH + 1];
 #ifdef DEBUG
         oer_debug(OER_DEBUG_INFO, "parsemsg(\"%s\", \"%s\", \"%s\", \"%s\")\n", nick, userhost, target, message);
 #endif
@@ -1195,12 +1209,10 @@ void parsemsg(char *nick, char *userhost, char *target, char *message)
 	/* check for command */
         ppos = 0;
         is_command = 0;
-	if((ppos = parse(message, ppos, " ", np, NICKLEN, 0)) >= 0) {
-		/* it is possible that parse() will fail if message
-		   has just one word and it is longer than NICKLEN */
-                if(!strcasecmp(np, mystate->nick)) {
+	if((ppos = parse(message, ppos, " ", outstring, WRITE_BUFFER_LENGTH, 0)) >= 0) {
+                if(!strcasecmp(outstring, mystate->nick)) {
                         is_command = 1;
-                } else if(!strncasecmp(np, mystate->prefix, strlen(mystate->prefix))) {
+                } else if(!strncasecmp(outstring, mystate->prefix, strlen(mystate->prefix))) {
                         is_command = 1;
                         ppos = strlen(mystate->prefix);
                 }
@@ -1242,6 +1254,12 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 	int ppos;
 	int nppos;
         int prevppos;
+	int numofchans;
+        int numofadmins;
+        int numofusers;
+        int numofchanusers;
+        int numofadminmasks;
+        int numofusermasks;
 	time_t uptime_seconds;
 	time_t idle_seconds;
         time_t idle_per;
@@ -1261,8 +1279,10 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 	struct channel *this;
 	struct channel *that;
 	struct chanuser *u;
+	struct botuser *admin;
 	struct botuser *user;
 	struct channelban *cb;
+	struct maskstruct *ms;
 #ifdef DEBUG
         oer_debug(OER_DEBUG_INFO, "parsecommand(\"%s\", \"%s\", \"%s\", \"%s\")\n", target, nick, userhost, commandline);
 #endif
@@ -2527,6 +2547,71 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
                         delchannelban(this, cb->ban);
                 }
                 return;
+	case OER_WHICHCOMMAND_CYCLE:
+#ifdef DEBUG
+                oer_debug(OER_DEBUG_INFO, "parsecommand->CYCLE\n");
+#endif
+                if((this = getchptr(channel)) == NULL) {
+                        return;
+                }
+                if(!isadmin(this, userhost)) {
+                        return;
+                }
+#ifdef DEBUG
+                oer_debug(OER_DEBUG_INFO, "parsecommand)->cycling %s because %s!%s ordered me to\n", this->name, nick, userhost);
+#endif
+                snprintf(timed_str, WRITE_BUFFER_LENGTH, "PART %s", this->name);
+                timed_new(NULL, mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_HANDLING, timed_str);
+                return;
+	case OER_WHICHCOMMAND_BSTATS:
+#ifdef DEBUG
+                oer_debug(OER_DEBUG_INFO, "parsecommand->BSTATS\n");
+#endif
+                /* may be NULL */
+                this = getchptr(channel);
+                proceed = (isopa(userhost)) ? 1 : 0;
+                numofchans = 0;
+                numofadmins = 0;
+                numofusers = 0;
+                numofchanusers = 0;
+                numofadminmasks = 0;
+                numofusermasks = 0;
+                for(admin = mystate->admins; admin != NULL; admin = admin->next) {
+                        numofadmins++;
+                        for(ms = admin->firstmask; ms != NULL; ms = ms->next) {
+                                numofadminmasks++;
+                        }
+                }
+		for(that = mystate->channels; that != NULL; that = that->next) {
+			/* first gather statistics */
+                        numofchans++;
+                        for(u = that->nicks; u != NULL; u = u->next) {
+                                numofchanusers++;
+                        }
+                        for(user = that->users; user != NULL; user = user->next) {
+                                numofusers++;
+                                for(ms = user->firstmask; ms != NULL; ms = ms->next) {
+                                        numofusermasks++;
+                                }
+                        }
+                        if(haschanflags(that, "U") && !isatleastop(that, nick, userhost)) {
+                                continue;
+                        }
+                        if(isatleastopnow(that, nick, userhost)) {
+                                proceed = 1;
+                        }
+                }
+		/* check if message was sent to a channel and if it was
+                   is "s" flag set */
+                if(tochan && haschanflags(this, "s")) {
+                        proceed = 1;
+                }
+                if(!proceed) {
+                        return;
+                }
+		snprintf(outstring, WRITE_BUFFER_LENGTH, "%d %s (%d %s), %d %s (%d %s), %d channel %s and %d %s", numofadmins, (numofadmins == 1) ? "admin" : "admins", numofadminmasks, (numofadminmasks == 1) ? "mask" : "masks", numofusers, (numofusers == 1) ? "user" : "users", numofusermasks, (numofusermasks == 1) ? "mask" : "masks", numofchanusers, (numofchanusers == 1) ? "user" : "users", numofchans, (numofchans == 1) ? "channel" : "channels");
+                sendreply((tochan) ? channel : nick, tochan, 0, OER_TIMED_PRIORITY_NORMAL, outstring);
+                return;
         case OER_WHICHCOMMAND_INVALID:
 #ifdef DEBUG
 		oer_debug(OER_DEBUG_INFO, "parsecommand->invalid or malformed command %s\n", command);
